/*
    addcontactpage.cpp - Kopete's Add Contact GUI

    Copyright (c) 2002 by Duncan Mac-Vicar Prett <duncan@kde.org>

    Kopete    (c) 2002-2004 by the Kopete developers  <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This library is free software; you can redistribute it and/or         *
    * modify it under the terms of the GNU Lesser General Public            *
    * License as published by the Free Software Foundation; either          *
    * version 2 of the License, or (at your option) any later version.      *
    *                                                                       *
    *************************************************************************
*/

#include "addcontactpage.h"

AddContactPage::AddContactPage(QWidget *parent) : QWidget(parent)
{
}

AddContactPage::~AddContactPage()
{
}

// vim: set noet ts=4 sts=4 sw=4:
