// oscardebug.h

// Copyright (C)  2005  Matt Rogers <mattr@kde.org>

// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.

// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
// 02110-1301  USA

#ifndef OSCARDEBUG_H
#define OSCARDEBUG_H

//OSCAR debugging definitions

//uncomment to get debug output for user info parsing
//#define OSCAR_USERINFO_DEBUG

//uncomment to get excessive amounts of debug info from
//various places in the code
//#define OSCAR_EXCESSIVE_DEBUG

//uncomment to get packet dumps in the debug output
//#define OSCAR_PACKET_PARSING_DEBUG

#endif
