@import url(../main.css);

.inActionTime, .inActionMetacontact, .inActionMessage {
	color: #000000;
}

.outActionTime, .outActionMetacontact, .outActionMessage {
	color: #000000;
}

.InternalTime, .InternalDelim, .InternalMessage {
	color: #000000;
}