/*
 * privacyruledlg.h
 * Copyright (C) 2006  Remko Troncon
 * Copyright (C) 2008  Charles Connell <charles@connells.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
#ifndef PRIVACYRULEDLG_H
#define PRIVACYRULEDLG_H

#include <KDialog>

#include "ui_privacyrule.h"

class PrivacyListItem;

class PrivacyRuleDlg : public KDialog
{
	Q_OBJECT

public:
	PrivacyRuleDlg();
	
	void setRule(const PrivacyListItem&);
	PrivacyListItem rule() const;

protected Q_SLOTS:
	void type_selected(const QString&);

private:
	Ui::PrivacyRule ui_;
};

#endif
