@import url(../main.css);

.inContentTime, .inMetacontact {
	color: #000000;
}

.outContentTime, .outMetacontact {
	color: #000000;
}

.inActionTime, .inActionMetacontact, .inActionMessage {
	color: #000000;
}

.outActionTime, .outActionMetacontact, .outActionMessage {
	color: #000000;
}

.InternalTime, .InternalDelim, .InternalMessage {
	color: #000000;
}

